#ifndef _fpas__defines_h_
#define _fpas__defines_h_

typedef short fpas_int_t;
#define fpas_chr yyctx->chr
#define fpas_val yyctx->val
#define fpas_lval yyctx->lval
#define fpas_stack yyctx->stack
#define fpas_debug yyctx->debug
#define fpas_nerrs yyctx->nerrs
#define fpas_errflag yyctx->errflag
#define fpas_state yyctx->state
#define fpas_yyn yyctx->yyn
#define fpas_yym yyctx->yym
#define fpas_jump yyctx->jump
#line 2 "fpas.y"
/*
libfawk - A function-only AWK dialect (pascal compiler)

Copyright (c) 2019 Tibor 'Igor2' Palinkas. All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

  * 1. Redistributions of source code must retain the above copyright notice,
    this list of conditions and the following disclaimer.
  * 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.
  * 3. Neither the name of the copyright holder nor the names of its
    contributors may be used to endorse or promote products derived from this
    software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

Project page: http://repo.hu/projects/libfawk
Source code: svn://repo.hu/libfawk/trunk
Contact the author: http://igor2.repo.hu/contact.html
*/

#define fpas_ctx_t fawk_ctx_t

#include "config.h"
#include <stdlib.h>
#include <stdio.h>
#include <genht/hash.h>
#include "libfawk.h"
#include "parser.h"
#include "fpas.y.tab.h"
#include "parser_fpas.h"
#include "symtab.h"
#include "compile.h"
#include "execute.h"

#define PUSH_IP() fawk_push_num(ctx, ctx->code.used)
#define CURR_IP() ctx->code.used

#define pas_func_ret() \
	fawkc_addi(ctx, FAWKI_RET); \
	fawkc_addnum(ctx, ctx->compiler.numargs + (ctx->compiler.numfixedargs >= 0));

#line 61 "fpas.y"
typedef union {
	char *str;
	fawk_num_t num;
} fpas_tokunion_t;
typedef fpas_tokunion_t fpas_STYPE;


#define PT_NEQ 257
#define PT_GTEQ 258
#define PT_LTEQ 259
#define PT_NIL 260
#define PT_IF 261
#define PT_THEN 262
#define PT_ELSE 263
#define PT_DO 264
#define PT_WHILE 265
#define PT_REPEAT 266
#define PT_UNTIL 267
#define PT_FOR 268
#define PT_DOWN 269
#define PT_TO 270
#define PT_FUNCTION 271
#define PT_BEGIN 272
#define PT_END 273
#define PT_CURRFUNC 274
#define PT_EXIT 275
#define PT_ID 276
#define PT_STRING 277
#define PT_NUM 278
#define PT_OR 279
#define PT_AND 280
#define PT_IN 281
#define PT_NOT 282
#define fpas_ERRCODE 256

#ifndef fpas_INITSTACKSIZE
#define fpas_INITSTACKSIZE 200
#endif

typedef struct {
	unsigned stacksize;
	fpas_int_t *s_base;
	fpas_int_t *s_mark;
	fpas_int_t *s_last;
	fpas_STYPE *l_base;
	fpas_STYPE *l_mark;
#if fpas_DEBUG
	int debug;
#endif
} fpas_STACKDATA;

typedef struct {
	int errflag;
	int chr;
	fpas_STYPE val;
	fpas_STYPE lval;
	int nerrs;
	int yym, yyn, state;
	int jump;
	int stack_max_depth;
	int debug;

	/* variables for the parser stack */
	fpas_STACKDATA stack;
} fpas_yyctx_t;

typedef enum { fpas_RES_NEXT, fpas_RES_DONE, fpas_RES_ABORT } fpas_res_t;

FAWK_API int fpas_parse_init(fpas_yyctx_t *yyctx);
FAWK_API fpas_res_t fpas_parse(fpas_yyctx_t *yyctx, fpas_ctx_t *ctx, int tok, fpas_STYPE *lval);
FAWK_API void fpas_error(fpas_ctx_t *ctx, fpas_STYPE tok, const char *msg);


#endif /* _fpas__defines_h_ */
